@@ -2,7 +2,7 @@
 This file is part of the `arabluatex' package
 
 ArabLuaTeX -- Processing ArabTeX notation under LuaLaTeX
-Copyright (C) 2016  Robert Alessi
+Copyright (C) 2016--2017  Robert Alessi
 
 Please send error reports and suggestions for improvements to Robert
 Alessi <alessi@robertalessi.net>
@@ -33,14 +33,14 @@ hamzafv = {
    {a="i\"'", b="إ"},
    {a="y\"'", b="ئ"},
    {a="ؤ([^uaiUAI])", b="ؤْ%1"},
-   {a="ؤ$", b="ؤْ"},
-   {a="ؤ(%s)", b="ؤْ%1"},
+   {a="ؤ(%p?)$", b="ؤْ%1"},
+   {a="ؤ(%p?%s)", b="ؤْ%1"},
    {a="أ([^uaiUAI])", b="أْ%1"},
-   {a="أ$", b="أْ"},
-   {a="أ(%s)", b="أْ%1"},
+   {a="أ(%p?)$", b="أْ%1"},
+   {a="أ(%p?%s)", b="أْ%1"},
    {a="ئ([^uaiUAI])", b="ئْ%1"},
-   {a="ئ$", b="ئْ"},
-   {a="ئ(%s)", b="ئْ%1"},
+   {a="ئ(%p?)$", b="ئْ%1"},
+   {a="ئ(%p?%s)", b="ئْ%1"},
    -- hamza takes tašdīd too
    {a="''([Uu])", b="ؤؤ%1"},
    {a="''([Aa])", b="أأ%1"},
@@ -58,14 +58,24 @@ hamzafv = {
    {a="([^uiUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
-   {a="(A)(')(uN?)$", b="aآء%3"},
-   {a="(A)(')(uN?)(%W)", b="aآء%3%4"},
-   {a="(A)(')(iN?)$", b="aآء%3"},
-   {a="(A)(')(iN?)(%W)", b="aآء%3%4"},
+   {a="(A)(')(uN?%p?)$", b="aآء%3"},
+   {a="(A)(')(uN?)(%p?%s)", b="aآء%3%4"},
+   {a="(A)(')(iN?%p?)$", b="aآء%3"},
+   {a="(A)(')(iN?)(%p?%s)", b="aآء%3%4"},
    {a="(A)(')([iI])", b="aآئ%3"}, -- historic madda
    {a="(A)(')(u)", b="aآؤ%3"}, -- historic madda
    {a="(A)(')", b="aآء"}, -- historic madda
    -- initial (needs both ^ and %W patterns)
+   -- 'aw: the diphthong is to be resolved into 'awi' (next 8 lines)
+   {a="^('aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(%W)('aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
+   {a="^('aw)(o)([%(%[%|%<]?\"?[uai])([%S]-o)", b="%1i"},
+   {a="(%W)('aw)(o)([%(%[%|%<]?\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
+   {a="(%W)('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
+   {a="^('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"},
+   {a="(%W)('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"},
+   -- then the 'initial' rules for the remaining cases
    {a="^(')([ua])", b="أ%2"},
    {a="^(')(i)", b="إ%2"},
    {a="(%W)(')([ua])", b="%1أ%3"},
@@ -74,31 +84,33 @@ hamzafv = {
    -- ^say'aN and .zim'aN are special orthographies
    {a="(%^say)(%')(aN)", b="%1ئ%3"},
    {a="(.zi?m)(%')(aN)", b="%1ئ%3"},
-   {a="([^uai])(')([uai]N?)$", b="%1ء%3"},
-   {a="([^uai])(')([uai]N?)(%W)", b="%1ء%3%4"},
+   {a="(')(I%p?)$", b="إ%2"}, -- affix pr. 1st person sg.
+   {a="(')(I)(%p?%s)", b="إ%2%3"}, -- affix pr. 1st person sg.
+   {a="([^uai])(')([uai]N?%p?)$", b="%1ء%3"},
+   {a="([^uai])(')([uai]N?)(%p?%s)", b="%1ء%3%4"},
 -- u
-   {a="(u)(')([uai]N?)$", b="%1ؤ%3"},
-   {a="(u)(')([uai]N?)(%W)", b="%1ؤ%3%4"},
-   {a="(u)(')$", b="%1ؤْ"},
-   {a="(u)(')(%W)", b="%1ؤْ%3"},
+   {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
+   {a="(u)('%p?)$", b="%1ؤْ"},
+   {a="(u)(')(%p?%s)", b="%1ؤْ%3"},
 -- a
-   {a="(a)(')(A)$", b="%1آ"},
-   {a="(a)(')(A)(%W)", b="%1آ%4"},
-   {a="(a)(')([u]N?)$", b="%1أ%3"},
-   {a="(a)(')([u]N?)(%W)", b="%1أ%3%4"},
-   {a="(a)(')(a)$", b="%1أ%3"},
-   {a="(a)(')(a)(%W)", b="%1أ%3%4"},
-   {a="(a)(')(aN)$", b="%1أً"},
-   {a="(a)(')(aN)(%W)", b="%1أً%4"},
-   {a="(a)(')([i]N?)$", b="%1إ%3"},
-   {a="(a)(')([i]N?)(%W)", b="%1إ%3%4"},
-   {a="(a)(')$", b="%1أْ"},
-   {a="(a)(')(%W)", b="%1أْ%3"},
+   {a="(a)(')(A%p?)$", b="%1آ"},
+   {a="(a)(')(A)(%p?%s)", b="%1آ%4"},
+   {a="(a)(')([u]N?%p?)$", b="%1أ%3"},
+   {a="(a)(')([u]N?)(%p?%s)", b="%1أ%3%4"},
+   {a="(a)(')(a%p?)$", b="%1أ%3"},
+   {a="(a)(')(a)(%p?%s)", b="%1أ%3%4"},
+   {a="(a)(')(aN%p?)$", b="%1أً"},
+   {a="(a)(')(aN)(%p?%s)", b="%1أً%4"},
+   {a="(a)(')([i]N?%p?)$", b="%1إ%3"},
+   {a="(a)(')([i]N?)(%p?%s)", b="%1إ%3%4"},
+   {a="(a)('%p?)$", b="%1أْ"},
+   {a="(a)(')(%p?%s)", b="%1أْ%3"},
 -- i
-   {a="(i)(')([uai]N?)$", b="%1ئ%3"},
-   {a="(i)(')([uai]N?)(%W)", b="%1ئ%3%4"},
-   {a="(i)(')$", b="%1ئْ"},
-   {a="(i)(')(%W)", b="%1ئْ%3"},
+   {a="(i)(')([uai]N?%p?)$", b="%1ئ%3"},
+   {a="(i)(')([uai]N?)(%p?%s)", b="%1ئ%3%4"},
+   {a="(i)('%p?)$", b="%1ئْ"},
+   {a="(i)(')(%p?%s)", b="%1ئْ%3"},
 --
    -- middle
    {a="(U)(')", b="%1ء"},
@@ -131,14 +143,14 @@ hamzafveasy = { -- differences marked below with 'easy'
    {a="i\"'", b="إ"},
    {a="y\"'", b="ئ"},
    {a="ؤ([^uaiUAI])", b="ؤْ%1"},
-   {a="ؤ$", b="ؤْ"},
-   {a="ؤ(%s)", b="ؤْ%1"},
+   {a="ؤ(%p?)$", b="ؤْ%1"},
+   {a="ؤ(%p?%s)", b="ؤْ%1"},
    {a="أ([^uaiUAI])", b="أْ%1"},
-   {a="أ$", b="أْ"},
-   {a="أ(%s)", b="أْ%1"},
+   {a="أ(%p?)$", b="أْ%1"},
+   {a="أ(%p?%s)", b="أْ%1"},
    {a="ئ([^uaiUAI])", b="ئْ%1"},
-   {a="ئ$", b="ئْ"},
-   {a="ئ(%s)", b="ئْ%1"},
+   {a="ئ(%p?)$", b="ئْ%1"},
+   {a="ئ(%p?%s)", b="ئْ%1"},
    -- hamza takes tašdīd too
    {a="''([Uu])", b="ؤؤ%1"},
    {a="''([Aa])", b="أأ%1"},
@@ -157,15 +169,25 @@ hamzafveasy = { -- differences marked below with 'easy'
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    --easy (begin)
-   {a="(A)(')(uN?)$", b="aاء%3"},
-   {a="(A)(')(uN?)(%W)", b="aاء%3%4"},
-   {a="(A)(')(iN?)$", b="aاء%3"},
-   {a="(A)(')(iN?)(%W)", b="aاء%3%4"},
+   {a="(A)(')(uN?%p?)$", b="aاء%3"},
+   {a="(A)(')(uN?)(%p?%s)", b="aاء%3%4"},
+   {a="(A)(')(iN?%p?)$", b="aاء%3"},
+   {a="(A)(')(iN?)(%p?%s)", b="aاء%3%4"},
    {a="(A)(')([iI])", b="aائ%3"}, -- historic madda
    {a="(A)(')(u)", b="aاؤ%3"}, -- historic madda
    {a="(A)(')", b="aاء"}, -- historic madda
    --easy (end)
    -- initial (needs both ^ and %W patterns)
+   -- 'aw: the diphthong is to be resolved into 'awi' (next 8 lines)
+   {a="^('aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(%W)('aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
+   {a="^('aw)(o)([%(%[%|%<]?\"?[uai])([%S]-o)", b="%1i"},
+   {a="(%W)('aw)(o)([%(%[%|%<]?\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
+   {a="(%W)('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
+   {a="^('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"},
+   {a="(%W)('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"},
+   -- then the 'initial' rules for the remaining cases
    {a="^(')([ua])", b="أ%2"},
    {a="^(')(i)", b="إ%2"},
    {a="(%W)(')([ua])", b="%1أ%3"},
@@ -174,31 +196,33 @@ hamzafveasy = { -- differences marked below with 'easy'
    -- ^say'aN and .zim'aN are special orthographies
    {a="(%^say)(%')(aN)", b="%1ئ%3"},
    {a="(.zi?m)(%')(aN)", b="%1ئ%3"},
-   {a="([^uai])(')([uai]N?)$", b="%1ء%3"},
-   {a="([^uai])(')([uai]N?)(%W)", b="%1ء%3%4"},
+   {a="(')(I%p?)$", b="إ%2"}, -- affix pr. 1st person sg.
+   {a="(')(I)(%p?%s)", b="إ%2%3"}, -- affix pr. 1st person sg.
+   {a="([^uai])(')([uai]N?%p?)$", b="%1ء%3"},
+   {a="([^uai])(')([uai]N?)(%p?%s)", b="%1ء%3%4"},
 -- u
-   {a="(u)(')([uai]N?)$", b="%1ؤ%3"},
-   {a="(u)(')([uai]N?)(%W)", b="%1ؤ%3%4"},
-   {a="(u)(')$", b="%1ؤْ"},
-   {a="(u)(')(%W)", b="%1ؤْ%3"},
+   {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
+   {a="(u)('%p?)$", b="%1ؤْ"},
+   {a="(u)(')(%p?%s)", b="%1ؤْ%3"},
 -- a
-   {a="(a)(')(A)$", b="%1آ"},
-   {a="(a)(')(A)(%W)", b="%1آ%4"},
-   {a="(a)(')([u]N?)$", b="%1أ%3"},
-   {a="(a)(')([u]N?)(%W)", b="%1أ%3%4"},
-   {a="(a)(')(a)$", b="%1أ%3"},
-   {a="(a)(')(a)(%W)", b="%1أ%3%4"},
-   {a="(a)(')(aN)$", b="%1أً"},
-   {a="(a)(')(aN)(%W)", b="%1أً%4"},
-   {a="(a)(')([i]N?)$", b="%1إ%3"},
-   {a="(a)(')([i]N?)(%W)", b="%1إ%3%4"},
-   {a="(a)(')$", b="%1أْ"},
-   {a="(a)(')(%W)", b="%1أْ%3"},
+   {a="(a)(')(A%p?)$", b="%1آ"},
+   {a="(a)(')(A)(%p?%s)", b="%1آ%4"},
+   {a="(a)(')([u]N?%p?)$", b="%1أ%3"},
+   {a="(a)(')([u]N?)(%p?%s)", b="%1أ%3%4"},
+   {a="(a)(')(a%p?)$", b="%1أ%3"},
+   {a="(a)(')(a)(%p?%s)", b="%1أ%3%4"},
+   {a="(a)(')(aN%p?)$", b="%1أً"},
+   {a="(a)(')(aN)(%p?%s)", b="%1أً%4"},
+   {a="(a)(')([i]N?%p?)$", b="%1إ%3"},
+   {a="(a)(')([i]N?)(%p?%s)", b="%1إ%3%4"},
+   {a="(a)('%p?)$", b="%1أْ"},
+   {a="(a)(')(%p?%s)", b="%1أْ%3"},
 -- i
-   {a="(i)(')([uai]N?)$", b="%1ئ%3"},
-   {a="(i)(')([uai]N?)(%W)", b="%1ئ%3%4"},
-   {a="(i)(')$", b="%1ئْ"},
-   {a="(i)(')(%W)", b="%1ئْ%3"},
+   {a="(i)(')([uai]N?%p?)$", b="%1ئ%3"},
+   {a="(i)(')([uai]N?)(%p?%s)", b="%1ئ%3%4"},
+   {a="(i)('%p?)$", b="%1ئْ"},
+   {a="(i)(')(%p?%s)", b="%1ئْ%3"},
 --
    -- middle
    {a="(U)(')", b="%1ء"},
@@ -220,7 +244,15 @@ hamzafveasy = { -- differences marked below with 'easy'
    {a="(i)(')([^uaiUAI])", b="%1ئْ%3"}
 }
 
-tanwinfv = {
+tanwinfv = { -- with assimilations (\SetArbDflt*)
+   -- assimilations (begin)
+   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([rlmnwy])", b="%4%4"},
+   {a="(%-?[uai]NU)(%s)([rlmnwy])", b="%1%2%3%3"},
+   -- assimilations (end)
+   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([uai])", b="ٱ"},
+   {a="%-?(uNU)(%s)([uai])", b="%1%2ٱ"},
+   {a="%-?(aNU)(%s)([uai])", b="%1%2ٱ"},
+   {a="%-?(iNU)(%s)([uai])", b="%1%2ٱ"},
    {a="%-?uNU", b="ٌو"},
    {a="%-?aNU", b="ًوا"},
    {a="%-?iNU", b="ٍو"},
@@ -228,6 +260,8 @@ tanwinfv = {
    {a="%-?(aN[%_]?[AY])(%s)([uai])", b="%1%2ٱ"},
    -- assimilations (begin)
    {a="%-?(uN)(%s)([rlmnwy])", b="ٌ%2%3%3"},
+   {a="(o[%S]-)(%-?aN)(_A)(o)([rlmnwy])", b="%5%5"},
+   {a="(o[%S]-)(%-?aN)(Y)(o)([rlmnwy])", b="%5%5"},
    {a="%-?(aN)(_A)(%s)([rlmnwy])", b="ًى%3%4%4"},
    {a="%-?(aN)(Y)(%s)([rlmnwy])", b="ًى%3%4%4"},
    {a="(T)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
@@ -235,6 +269,8 @@ tanwinfv = {
    {a="([^TA])%-?(aN)(%s)([rlmnwy])", b="%1ًا%3%4%4"},
    {a="%-?(iNI?)(%s)([rlmnwy])", b="ٍ%2%3%3"},
    -- assimilations (end)
+   {a="(o[%S]-)(%-?aN)(_A)(o)([uai])", b="ٱ"},
+   {a="(o[%S]-)(%-?aN)(Y)(o)([uai])", b="ٱ"},
    -- quoted tanwīn (begin)
    {a="%-?(\"uN)", b=""},
    {a="(B)%-?(\"aN)", b="%1"},
@@ -256,6 +292,14 @@ tanwinfv = {
 }
 
 tanwinfveasy = { -- no assimilations (see below)
+   -- assimilations (begin)
+--easy   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([rlmnwy])", b="%4%4"},
+--easy   {a="(%-?[uai]NU)(%s)([rlmnwy])", b="%1%2%3%3"},
+   -- assimilations (end)
+   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([uai])", b="ٱ"},
+   {a="%-?(uNU)(%s)([uai])", b="%1%2ٱ"},
+   {a="%-?(aNU)(%s)([uai])", b="%1%2ٱ"},
+   {a="%-?(iNU)(%s)([uai])", b="%1%2ٱ"},
    {a="%-?uNU", b="ٌو"},
    {a="%-?aNU", b="ًوا"},
    {a="%-?iNU", b="ٍو"},
@@ -263,6 +307,8 @@ tanwinfveasy = { -- no assimilations (see below)
    {a="%-?(aN[%_]?[AY])(%s)([uai])", b="%1%2ٱ"},
    -- assimilations (begin)
 --easy   {a="%-?(uN)(%s)([rlmnwy])", b="ٌ%2%3%3"},
+--easy   {a="(o[%S]-)(%-?aN)(_A)(o)([rlmnwy])", b="%5%5"},
+--easy   {a="(o[%S]-)(%-?aN)(Y)(o)([rlmnwy])", b="%5%5"},
 --easy   {a="%-?(aN)(_A)(%s)([rlmnwy])", b="ًى%3%4%4"},
 --easy   {a="%-?(aN)(Y)(%s)([rlmnwy])", b="ًى%3%4%4"},
 --easy   {a="(T)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
@@ -270,6 +316,8 @@ tanwinfveasy = { -- no assimilations (see below)
 --easy   {a="([^TA])%-?(aN)(%s)([rlmnwy])", b="%1ًا%3%4%4"},
 --easy   {a="%-?(iNI?)(%s)([rlmnwy])", b="ٍ%2%3%3"},
    -- assimilations (end)
+   {a="(o[%S]-)(%-?aN)(_A)(o)([uai])", b="ٱ"},
+   {a="(o[%S]-)(%-?aN)(Y)(o)([uai])", b="ٱ"},
    -- quoted tanwīn (begin)
    {a="%-?(\"uN)", b=""},
    {a="(B)%-?(\"aN)", b="%1"},
@@ -293,25 +341,37 @@ tanwinfveasy = { -- no assimilations (see below)
 trigraphsfv = { -- trigraphs or more
    -- 'llatI / 'llad_I
    {a="^'ll(a)([%_]?[dt])", b="ٱلّ%1%2"},
-   {a="([%s%-])'ll(a)([%_]?[dt])", b="%1ٱلّ%2%3"},
+   {a="([%(%[%|%<%s%-])'ll(a)([%_]?[dt])", b="%1ٱلّ%2%3"}, --p
+   -- law: the diphthong is to be resolved into 'awi' (next 8 lines)
+   {a="^(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(%W)(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
+   {a="^(law)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(%W)(law)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
+   {a="(%W)(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
+   {a="^(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"},
+   {a="(%W)(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"},
    -- al- + lām
    {a="^(a)l%-(l)", b="ا%1ل%2%2"},
-   {a="([%s%-])(a)l%-(l)", b="%1ا%2ل%3%3"},
+   {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1ا%2ل%3%3"}, --p
    -- al- + solar consonant
    {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="ا%1ل%2%2"},
-   {a="([%s%-])(a)l%-([%_%^%.]?[tdrzsn])", b="%1ا%2ل%3%3"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsn])", b="%1ا%2ل%3%3"}, --p
    -- assim. art. + solar consonant
    {a="^(a)([%_%^%.]?[tdrzsn])%-", b="ا%1ل%2"},
-   {a="([%s%-])(a)([%_%^%.]?[tdrzsn])%-", b="%1ا%2ل%3"},
+   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsn])%-", b="%1ا%2ل%3"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\"?[uai])", b="ا%1ل%2ٱ"},
-   {a="([%s%-])(a)l%-(\"?[uai])", b="%1ا%2ل%3ٱ"},
+   {a="([%(%[%|%<%s%-o])(a)l%-(\"?[uai])", b="%1ا%2ل%3ٱ"}, --p
    -- li-/la + art. + initial unstable hamza is a special orthography
    {a="l([ai])%-l%-(\"?[uai])", b="ل%1ل%2ٱ"},
    -- al- + lunar consonant (i.e. what remains)
    {a="^(a)l%-", b="ا%1لْ"},
-   {a="([%s%-])(a)l%-", b="%1ا%2لْ"},
+   {a="([%(%[%|%<%s%-o])(a)l%-", b="%1ا%2لْ"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
+   {a="(aW)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="awuا"},
+   {a="(aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1u"},
+   {a="(ay)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
    {a="(aW)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="awuا%2%3"},
    {a="(aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1u%2%3"},
    {a="(ay)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
@@ -332,42 +392,55 @@ trigraphsfv = { -- trigraphs or more
    -- art. with waṣla + lunar consonant (i.e. what remains)
    {a="'l%-", b="ٱلْ"},
    -- the silent wāw
-   {a="uU$", b="uو"},
-   {a="uU(%W)", b="uو%1"},
-   {a="aU$", b="aو"},
-   {a="aU(%W)", b="aو%1"},
-   {a="iU$", b="iو"},
-   {a="iU(%W)", b="iو%1"},
+   {a="uU(%p?)$", b="uو%1"},
+   {a="uU(%p?%s)", b="uو%1"},
+   {a="aU(%p?)$", b="aو%1"},
+   {a="aU(%p?%s)", b="aو%1"},
+   {a="iU(%p?)$", b="iو%1"},
+   {a="iU(%p?%s)", b="iو%1"},
    -- words ending in -āT with silent wāw/yāʾ
    {a="(_a)UA", b="%1وا"},
    {a="(_a)U", b="%1و"},
-   {a="(_a)I", b="%1ي"},
-   -- assimilations
-   {a="(n)(%s)([rlmnwy])", b="%1%2%3%3"}
+   {a="(_a)I", b="%1ي"}
 }
 
+-- idgham/assimilation rules for trigraphs apply here : see voc lua
+-- file.
+
 trigraphsfveasy = { -- trigraphs or more (see 'easy' tag below for the diffs)
    -- 'llatI / 'llad_I
    {a="^'ll(a)([%_]?[dt])", b="ٱلّ%1%2"},
-   {a="([%s%-])'ll(a)([%_]?[dt])", b="%1ٱلّ%2%3"},
+   {a="([%(%[%|%<%s%-])'ll(a)([%_]?[dt])", b="%1ٱلّ%2%3"}, --p
+   -- law: the diphthong is to be resolved into 'awi' (next 8 lines)
+   {a="^(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(%W)(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
+   {a="^(law)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(%W)(law)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
+   {a="(%W)(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
+   {a="^(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"},
+   {a="(%W)(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"},
    -- al- + lām
    {a="^(a)l%-(l)", b="ا%1ل%2%2"},
-   {a="([%s%-])(a)l%-(l)", b="%1ا%2ل%3%3"},
+   {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1ا%2ل%3%3"}, --p
    -- al- + solar consonant
    {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="ا%1ل%2%2"},
-   {a="([%s%-])(a)l%-([%_%^%.]?[tdrzsn])", b="%1ا%2ل%3%3"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsn])", b="%1ا%2ل%3%3"}, --p
    -- assim. art. + solar consonant
    {a="^(a)([%_%^%.]?[tdrzsn])%-", b="ا%1ل%2"},
-   {a="([%s%-])(a)([%_%^%.]?[tdrzsn])%-", b="%1ا%2ل%3"},
+   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsn])%-", b="%1ا%2ل%3"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\"?[uai])", b="ا%1ل%2ٱ"},
-   {a="([%s%-])(a)l%-(\"?[uai])", b="%1ا%2ل%3ٱ"},
+   {a="([%(%[%|%<%s%-o])(a)l%-(\"?[uai])", b="%1ا%2ل%3ٱ"}, --p
    -- li-/la + art. + initial unstable hamza is a special orthography
    {a="l([ai])%-l%-(\"?[uai])", b="ل%1ل%2ٱ"},
    -- al- + lunar consonant (i.e. what remains)
    {a="^(a)l%-", b="ا%1لْ"},
-   {a="([%s%-])(a)l%-", b="%1ا%2لْ"},
+   {a="([%(%[%|%<%s%-o])(a)l%-", b="%1ا%2لْ"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
+   {a="(aW)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="awuا"},
+   {a="(aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1u"},
+   {a="(ay)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
    {a="(aW)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="awuا%2%3"},
    {a="(aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1u%2%3"},
    {a="(ay)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
@@ -388,21 +461,19 @@ trigraphsfveasy = { -- trigraphs or more (see 'easy' tag below for the diffs)
    -- art. with waṣla + lunar consonant (i.e. what remains)
    {a="'l%-", b="ٱلْ"},
    -- the silent wāw
-   {a="uU$", b="uو"},
-   {a="uU(%W)", b="uو%1"},
-   {a="aU$", b="aو"},
-   {a="aU(%W)", b="aو%1"},
-   {a="iU$", b="iو"},
-   {a="iU(%W)", b="iو%1"},
+   {a="uU(%p?)$", b="uو%1"},
+   {a="uU(%p?%s)", b="uو%1"},
+   {a="aU(%p?)$", b="aو%1"},
+   {a="aU(%p?%s)", b="aو%1"},
+   {a="iU(%p?)$", b="iو%1"},
+   {a="iU(%p?%s)", b="iو%1"},
    -- words ending in -āT with silent wāw/yāʾ
    {a="(_a)UA", b="%1وا"},
    {a="(_a)U", b="%1و"},
-   {a="(_a)I", b="%1ي"},
-   -- assimilations
---easy   {a="(n)(%s)([rlmnwy])", b="%1%2%3%3"}
+   {a="(_a)I", b="%1ي"}
 }
 
-digraphsfv = {
+digraphsfvidgham = {
    -- iʿrāb: straight double quote must be discarded
    {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
    {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
@@ -417,16 +488,22 @@ digraphsfv = {
    -- ʾiʿrāb (end)
    -- initial straight double quote gives a connective ʾalif
    {a="^\"[uai]", b="ٱ"},
-   {a="([%s%-])\"[uai]", b="%1ٱ"},
+   {a="([%(%[%|%<%s%-])\"[uai]", b="%1ٱ"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
-   {a="(aW)(%s)(\"?[uai])", b="awuا%2ٱ"},
-   {a="(aw)(%s)(\"?[uai])", b="%1u%2ٱ"},
-   {a="(ay)(%s)(\"?[uai])", b="%1i%2ٱ"},
-   {a="([uai]%-)(\"?[uai])", b="%1ٱ"}, -- hyphen + initial alif without hamza
-   -- initial alif without hamza
-   {a="([%_]?[uaiUAIY])(%s)(\"?[uai])", b="%1%2ٱ"},
-   {a="^([uai])", b="ا%1"},      -- initial alif without hamza
-   {a="(%s)([uai])", b="%1ا%2"}, -- initial alif without hamza
+   {a="(aW)(o)(\"?[uai])([%S]-o)", b="awuا"},
+   {a="(aW)(%s)([%(%[%|%<]?)(\"?[uai])", b="awuا%2%3ٱ"}, --p
+   {a="(aw)(o)(\"?[uai])([%S]-o)", b="%1u"},
+   {a="(aw)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1u%2%3ٱ"}, --p
+   {a="(ay)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3ٱ"}, --p
+   -- hyphen + initial alif without hamza
+   --   {a="([uai]%-)(\"?[uai])", b="%1ٱ"},
+   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)(%a)", b="%1ٱ%3%4"},
+   -- initial alif without hamza (next 4 lines)
+   {a="(o[%S]-)([uaiUAIY])(o)(\"?[uai])", b="ٱ"},
+   {a="([%_]?[uaiUAIY])(%s[%(%[%|%<]?)(\"?[uai])", b="%1%2ٱ"}, --p
+   {a="^([%(%[%|%<]?)(\"?[uai])", b="%1ا%2"}, --p
+   {a="(%s)([%(%[%|%<]?)(\"?[uai])", b="%1%2ا%3"}, --p
    {a="%-%-", b="ـ"},
    {a="ؤؤ", b="ؤّ"},
    {a="أأ", b="أّ"},
@@ -438,6 +515,7 @@ digraphsfv = {
    {a="%_t%_t", b="ثّ"},
    {a="jj", b="جّ"},
    {a="%^g%^g", b="جّ"},
+   {a="%.h%.h", b="حّ"},
    {a="xx", b="خّ"},
    {a="%_h%_h", b="خّ"},
    {a="dd", b="دّ"},
@@ -464,8 +542,8 @@ digraphsfv = {
    -- sukūn begin
    -- first, take out hyphen if any:
    {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])%-([uaiUAI])", b="%1%2"},
-   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])$", b="%1ْ"},
-   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%s])", b="%1ْ%2"},
+   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy]%p?)$", b="%1ْ"},
+   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?%s)", b="%1ْ%2"},
    {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%_]?[^%_uaiUAIYًٌٍ])", b="%1ْ%2"},
    -- take out sukūn in cases of assimilation
    {a="(n)(ْ)(%s)(ر)", b="%1%3%4"},
@@ -503,6 +581,114 @@ digraphsfv = {
    {a="%^d", b="ڊ"}
 }
 
+digraphsfv = {
+   -- iʿrāb: straight double quote must be discarded
+   {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p?%s)", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p?)$", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p?%s)", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p?)$", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p?%s)", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p?)$", b="%2%3"},
+   {a="(%-)(\"?[uai])(%p?%s)", b="%2%3"},
+   {a="(%-)(\"?[uai])(%p?)$", b="%2%3"},
+   -- ʾiʿrāb (end)
+   -- initial straight double quote gives a connective ʾalif
+   {a="^\"[uai]", b="ٱ"},
+   {a="([%(%[%|%<%s%-])\"[uai]", b="%1ٱ"}, --p
+   -- diphthongs to be resolved before ʾalif conjunctionis
+   {a="(aW)(o)(\"?[uai])([%S]-o)", b="awuا"},
+   {a="(aW)(%s)([%(%[%|%<]?)(\"?[uai])", b="awuا%2%3ٱ"}, --p
+   {a="(aw)(o)(\"?[uai])([%S]-o)", b="%1u"},
+   {a="(aw)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1u%2%3ٱ"}, --p
+   {a="(ay)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3ٱ"}, --p
+   -- hyphen + initial alif without hamza
+   --   {a="([uai]%-)(\"?[uai])", b="%1ٱ"},
+   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)(%a)", b="%1ٱ%3%4"},
+   -- initial alif without hamza (next 4 lines)
+   {a="(o[%S]-)([uaiUAIY])(o)(\"?[uai])", b="ٱ"},
+   {a="([%_]?[uaiUAIY])(%s[%(%[%|%<]?)(\"?[uai])", b="%1%2ٱ"}, --p
+   {a="^([%(%[%|%<]?)(\"?[uai])", b="%1ا%2"}, --p
+   {a="(%s)([%(%[%|%<]?)(\"?[uai])", b="%1%2ا%3"}, --p
+   {a="%-%-", b="ـ"},
+   {a="ؤؤ", b="ؤّ"},
+   {a="أأ", b="أّ"},
+   {a="ئئ", b="ئّ"},
+   {a="bb", b="بّ"},
+   {a="BB", b="ـّ"},
+   {a="(%_)([thd])([thd])", b="%1%2|%3"},
+   {a="tt", b="تّ"},
+   {a="%_t%_t", b="ثّ"},
+   {a="jj", b="جّ"},
+   {a="%^g%^g", b="جّ"},
+   {a="%.h%.h", b="حّ"},
+   {a="xx", b="خّ"},
+   {a="%_h%_h", b="خّ"},
+   {a="dd", b="دّ"},
+   {a="%_d%_d", b="ذّ"},
+   {a="rr", b="رّ"},
+   {a="zz", b="زّ"},
+   {a="ss", b="سّ"},
+   {a="%^s%^s", b="شّ"},
+   {a="%.s%.s", b="صّ"},
+   {a="%.d%.d", b="ضّ"},
+   {a="%.t%.t", b="طّ"},
+   {a="%.z%.z", b="ظّ"},
+   {a="%`%`", b="عّ"},
+   {a="%.g%.g", b="غّ"},
+   {a="ff", b="فّ"},
+   {a="qq", b="قّ"},
+   {a="kk", b="كّ"},
+   {a="ll", b="لّ"},
+   {a="mm", b="مّ"},
+   {a="nn", b="نّ"},
+   {a="hh", b="هّ"},
+   {a="ww", b="وّ"},
+   {a="yy", b="يّ"},
+   -- sukūn begin
+   -- first, take out hyphen if any:
+   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])%-([uaiUAI])", b="%1%2"},
+   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy]%p?)$", b="%1ْ"},
+   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?%s)", b="%1ْ%2"},
+   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%_]?[^%_uaiUAIYًٌٍ])", b="%1ْ%2"},
+   -- take out sukūn in cases of assimilation
+--   {a="(n)(ْ)(%s)(ر)", b="%1%3%4"},
+--   {a="(n)(ْ)(%s)(و)", b="%1%3%4"},
+--   {a="(n)(ْ)(%s)(ي)", b="%1%3%4"},
+--   {a="(n)(ْ)(%s)(ل)", b="%1%3%4"},
+--   {a="(n)(ْ)(%s)(م)", b="%1%3%4"},
+--   {a="(n)(ْ)(%s)(ن)", b="%1%3%4"},
+   {a="ْ\"", b="\""},
+   -- sukūn end
+   {a="_t", b="ث"},
+   {a="%^g", b="ج"},
+   {a="%.h", b="ح"},
+   {a="_h", b="خ"},
+   {a="_d", b="ذ"},
+   {a="%^s", b="ش"},
+   {a="%.s", b="ص"},
+   {a="%.d", b="ض"},
+   {a="%.t", b="ط"},
+   {a="%.z", b="ظ"},
+   {a="%.g", b="غ"},
+   {a="(U)(A)", b="%1ا"},
+   {a="WA", b="وْا"},
+   {a="(a)W\"", b="%1وا"},
+   {a="(a)W", b="%1وْا"},
+   {a="_A", b="aى"},
+   {a="_u", b="ٗ"},
+   {a="_a", b="ٰ"},
+   {a="_i", b="ٖ"},
+   {a="%.b", b="ٮ"},
+   {a="%.f", b="ڡ"},
+   {a="%.q", b="ٯ"},
+   {a="%.k", b="ک"},
+   {a="%.n", b="ں"},
+   {a="%^d", b="ڊ"}
+}
+
 digraphsfveasy = { -- see the differences under 'easy' marker below
    -- iʿrāb: straight double quote must be discarded
    {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
@@ -518,16 +704,22 @@ digraphsfveasy = { -- see the differences under 'easy' marker below
    -- ʾiʿrāb (end)
    -- initial straight double quote gives a connective ʾalif
    {a="^\"[uai]", b="ٱ"},
-   {a="([%s%-])\"[uai]", b="%1ٱ"},
+   {a="([%(%[%|%<%s%-])\"[uai]", b="%1ٱ"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
-   {a="(aW)(%s)(\"?[uai])", b="awuا%2ٱ"},
-   {a="(aw)(%s)(\"?[uai])", b="%1u%2ٱ"},
-   {a="(ay)(%s)(\"?[uai])", b="%1i%2ٱ"},
-   {a="([uai]%-)(\"?[uai])", b="%1ٱ"}, -- hyphen + initial alif without hamza
-   -- initial alif without hamza
-   {a="([%_]?[uaiUAIY])(%s)(\"?[uai])", b="%1%2ٱ"},
-   {a="^([uai])", b="ا%1"},      -- initial alif without hamza
-   {a="(%s)([uai])", b="%1ا%2"}, -- initial alif without hamza
+   {a="(aW)(o)(\"?[uai])([%S]-o)", b="awuا"},
+   {a="(aW)(%s)([%(%[%|%<]?)(\"?[uai])", b="awuا%2%3ٱ"}, --p
+   {a="(aw)(o)(\"?[uai])([%S]-o)", b="%1u"},
+   {a="(aw)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1u%2%3ٱ"}, --p
+   {a="(ay)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3ٱ"}, --p
+   -- hyphen + initial alif without hamza
+   -- {a="([uai]%-)(\"?[uai])", b="%1ٱ"},
+   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)(%a)", b="%1ٱ%3%4"},
+   -- initial alif without hamza (next 4 lines)
+   {a="(o[%S]-)([uaiUAIY])(o)(\"?[uai])", b="ٱ"},
+   {a="([%_]?[uaiUAIY])(%s[%(%[%|%<]?)(\"?[uai])", b="%1%2ٱ"}, --p
+   {a="^([%(%[%|%<]?)(\"?[uai])", b="%1ا%2"}, --p
+   {a="(%s)([%(%[%|%<]?)(\"?[uai])", b="%1%2ا%3"}, --p
    {a="%-%-", b="ـ"},
    {a="ؤؤ", b="ؤّ"},
    {a="أأ", b="أّ"},
@@ -539,6 +731,7 @@ digraphsfveasy = { -- see the differences under 'easy' marker below
    {a="%_t%_t", b="ثّ"},
    {a="jj", b="جّ"},
    {a="%^g%^g", b="جّ"},
+   {a="%.h%.h", b="حّ"},
    {a="xx", b="خّ"},
    {a="%_h%_h", b="خّ"},
    {a="dd", b="دّ"},
@@ -568,11 +761,13 @@ digraphsfveasy = { -- see the differences under 'easy' marker below
    {a="ْ", b=""}, 
    -- first, take out hyphen if any:
 --   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])%-([uaiUAI])", b="%1%2"},
---   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])$", b="%1ْ"},
---   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%s])", b="%1ْ%2"},
+--   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy]%p?)$", b="%1ْ"},
+--   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?%s)", b="%1ْ%2"},
 --   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%_]?[^%_uaiUAIًٌٍ])", b="%1ْ%2"},
    -- take out sukūn in cases of assimilation
---   {a="(n)(ْ)(%s)([روي])", b="%1%3%4"},
+--   {a="(n)(ْ)(%s)(ر)", b="%1%3%4"},
+--   {a="(n)(ْ)(%s)(و)", b="%1%3%4"},
+--   {a="(n)(ْ)(%s)(ي)", b="%1%3%4"},
 --   {a="(n)(ْ)(%s)([ل])", b="%1%3%4"},
 --   {a="(n)(ْ)(%s)([م])", b="%1%3%4"},
 --   {a="(n)(ْ)(%s)([ن])", b="%1%3%4"},
